from re import findall

class filters:
	me                       = lambda bot, message: message["chat_updates"][-1]["chat"]["last_message"]["is_mine"]
	PV                       = lambda bot, message: message["message_updates"][-1]["type"] == "User"
	bot                      = lambda bot, message: message["message_updates"][-1]["type"] == "Bot"
	group                    = lambda bot, message: message["message_updates"][-1]["type"] == "Group"
	channel                  = lambda bot, message: message["message_updates"][-1]["type"] == "Channel"
	service                  = lambda bot, message: message["message_updates"][-1]["type"] == "Service"

	gif                      = lambda bot, message: message["message_updates"][-1]["message"]["file_inline"]["type"] == "Gif"
	poll                     = lambda bot, message: message["message_updates"][-1]["message"]["type"] == "Poll"
	quiz                     = lambda bot, message: message["message_updates"][-1]["message"]["type"] == "Poll3"
	text                     = lambda bot, message: message["message_updates"][-1]["message"]["type"] == "Text"
	media                    = lambda bot, message: "file_inline" in message["message_updates"][-1]["message"].keys()
	video                    = lambda bot, message: message["message_updates"][-1]["message"]["file_inline"]["type"] == "Video"
	photo                    = lambda bot, message: message["message_updates"][-1]["message"]["file_inline"]["type"] == "Image"
	voice                    = lambda bot, message: message["message_updates"][-1]["message"]["file_inline"]["type"] == "Voice"
	music                    = lambda bot, message: message["message_updates"][-1]["message"]["file_inline"]["type"] == "Music"
	rubino                   = lambda bot, message: message["message_updates"][-1]["message"]["type"] == "RubinoPost"
	sticker                  = lambda bot, message: message["message_updates"][-1]["message"]["type"] == "Sticker"
	document                 = lambda bot, message: message["message_updates"][-1]["message"]["file_inline"]["type"] == "File"

	hasGroupLink             = lambda bot, message: bool(findall(r"rubika.ir/joing/\w{32}", message["message_updates"][-1]["message"]["text"]))
	hasChannelLink           = lambda bot, message: bool(findall(r"rubika.ir/joinc/\w{32}", message["message_updates"][-1]["message"]["text"]))
	hasUsername              = lambda bot, message: any([bot.getInfoByUsername(i.replace("@","")).get("data").get("user") is not None for i in findall(r"@[\w|\d]+", message["message_updates"][-1]["message"]["text"])])

	bold                     = lambda bot, message: any([j["type"] == "Bold" for j in [i for i in message["message_updates"][-1]["message"]["metadata"]]])
	mono                     = lambda bot, message: any([j["type"] == "Mono" for j in [i for i in message["message_updates"][-1]["message"]["metadata"]]])
	italic                   = lambda bot, message: any([j["type"] == "Italic" for j in [i for i in message["message_updates"][-1]["message"]["metadata"]]])
	mention                  = lambda bot, message: any([j["type"] == "Mention" for j in [i for i in message["message_updates"][-1]["message"]["metadata"]]])
	metadata                 = lambda bot, message: "metadata" in message["message_updates"][-1]["message"]

	onMessageSent            = lambda bot, message: message["message_updates"][-1]["action"] == "New"
	onMessageEdited          = lambda bot, message: message["message_updates"][-1]["action"] == "Edit"
	onMessageReplied         = lambda bot, message: "reply_to_message_id" in message["message_updates"][-1]["message"].keys()
	onMessageDeleted         = lambda bot, message: message["message_updates"][-1]["action"] == "Delete"
	onMessageForwarded       = lambda bot, message: "forwarded_from" in message["message_updates"][-1]["message"].keys()
	onMessageForwardedNoLink = lambda bot, message: "forwarded_no_link" in message["message_updates"][-1]["message"].keys()

	event                    = lambda bot, message: message["message_updates"][-1]["message"]["type"] == "Event"
	onMemberAdded            = lambda bot, message: message["message_updates"][-1]["message"]["event_data"]["type"] == "AddedGroupMembers"
	onMemberJoined           = lambda bot, message: message["message_updates"][-1]["message"]["event_data"]["type"] == "JoinedGroupByLink"
	onMemberLeaved           = lambda bot, message: message["message_updates"][-1]["message"]["event_data"]["type"] == "LeaveGroup"
	onMemberRemoved          = lambda bot, message: message["message_updates"][-1]["message"]["event_data"]["type"] == "RemoveGroupMembers"
	onMessagePinned          = lambda bot, message: message["message_updates"][-1]["message"]["event_data"]["type"] == "PinnedMessageUpdated"
	onVoiceChatStarted       = lambda bot, message: message["message_updates"][-1]["message"]["event_data"]["type"] == "VoiceChatFinished"
	onVoiceChatFinished      = lambda bot, message: message["message_updates"][-1]["message"]["event_data"]["type"] == "VoiceChatStarted"

class types: ALL, ANY, INVERT = "all", "any", "invert"


from random  import randint
from pathlib import Path

from rubika.configs import makeRubinoData, welcome

__version__   = "6.0.0"
__license__   = "GPLv3 license"
__copyright__ = "Copyright (C) 2022 Bahman Ahmadi <github.com/Bahman-Ahmadi>"

class Rubino:
	def __init__(self, username:str, auth:str):
		self.auth, self.profileID = auth, makeRubinoData(auth, "isExistUsername", {"username": username.replace('@', '')})["data"]["profile"]["id"]
		welcome(f"rubika library version {__version__}\n{__copyright__}\n➜ docs : https://rubikalib.github.io\n\nswitching rubino...\n")

	addPost           = lambda self, caption, fileID, hashFile, thumbnailID, thumbnailHash, width, height, post_type="Picture", is_multi_file=False: makeRubinoData(self.auth, "addPost", {"caption": caption, "file_id": fileID, "hash_file_receive": hashFile, "height": str(height), "width": str(width), "is_multi_file": is_multi_file, "post_type": post_type, "rnd": randint(100000, 999999999), "thumbnail_file_id": thumbnailID, "thumbnail_hash_file_receive": thumbnailHash, "profile_id": self.profileID})
	addStory          = lambda self, duration, hashFile, fileID, storyType, thumbnailID, thumbnailHash, width, height: makeRubinoData(self.auth, "addStory", {"duration": duration, "file_id": fileID, "hash_file_receive": hashFile, "height": height, "profile_id": self.profileID, "rnd": randint(100000, 999999999), "story_type": storyType, "thumbnail_file_id": thumbnailID, "thumbnail_hash_file_receive": thumbnailHash, "width": width})

	follow            = lambda self, followee_id: makeRubinoData(self.auth, "requestFollow", {"f_type": "Follow", "followee_id": followee_id, "profile_id": self.profileID})
	
	getStories        = lambda self, targetProfileID, limit=100                      : makeRubinoData(self.auth, "getProfileStories", {"limit": limit, "profile_id": targetProfileID})
	getProfilePosts   = lambda self, targetProfileID, limit=51, sort="FromMax"       : makeRubinoData(self.auth, "getProfilePosts", {"equal": False, "limit": limit, "sort": sort, "target_profile_id": targetProfileID, "profile_id": self.profileID})
	getRecentPosts    = lambda self, equal=False, limit=30, sort="FromMax"           : makeRubinoData(self.auth, "getRecentFollowingPosts", {"equal": equal, "limit": limit, "sort": sort, "profile_id": self.profileID})
	getComments       = lambda self, postID, postProfileID, limit=50, sort="FromMax" : makeRubinoData(self.auth, "getComments", {"equal": False, "limit": limit, "sort": sort, "post_id": postID, "profile_id": self.profileID, "post_profile_id": postProfileID})

	isExist           = lambda self, username: makeRubinoData(self.auth, "isExistUsername", {"username": username.replace('@','')})

	like              = lambda self, post_id, post_profile_id: makeRubinoData(self.auth, "likePostAction", {"action_type": "Like", "post_id": post_id, "post_profile_id": post_profile_id, "profile_id": self.profileID})

	requestUploadFile = lambda self, file, size=None, Type="Picture": makeRubinoData(self.auth, "requestUploadFile", {"file_name": file.split("/")[-1], "file_size": size or Path(file).stat().st_size, "file_type": Type, "profile_id": self.profileID})

	unfollow          = lambda self, followee_id: makeRubinoData(self.auth, "requestFollow", {"f_type": "Unfollow", "followee_id": followee_id, "profile_id": self.profileID})
	unlike            = lambda self, post_id, post_profile_id: makeRubinoData(self.auth, "likePostAction", {"action_type": "Unlike", "post_id": post_id, "post_profile_id": post_profile_id, "profile_id": self.profileID})
	updateProfile     = lambda self, name=None, bio=None, email=None: makeRubinoData(self.auth, "updateProfile", {"name": name, "bio": bio, "email": email})
	
	viewPost          = lambda self, post_id, post_profile_id: makeRubinoData(self.auth, "addPostViewCount", {"post_id": post_id, "post_profile_id": post_profile_id})


from re import findall
from threading import Thread
from datetime import datetime

bot = None
socket = None
lockTime = None
unlockTime = None

class Tools:
	def __init__(self, auth):
		from rubika.client import Bot, Socket
		bot = Bot("", auth=auth)
		socket = Socket(auth)

	def hasInsult(self, msg):
		return any(word in open("dontReadMe.txt").read().split("\n") for word in msg.split())
	
	def hasAD(self, msg):
		result = False
		links = list(map(lambda ID: ID.strip()[1:],findall(r"@[\w|_|\d]+", msg))) + list(map(lambda link:link.split("/")[-1],findall(r"rubika\.ir/\w+",msg)))
		joincORjoing = "joing" in msg or "joinc" in msg
	
		if joincORjoing: result = True
		else:
			for link in links:
				try:
					Type = bot.getInfoByUsername(link)["data"]["chat"]["abs_object"]["type"]
					if Type == "Channel": result = True
				except KeyError: result = False

		return result

	def hasSpam(self, messages):
		result, beforeIter = {"isExist":False, "spams":[]}, 0
		for Iter in range(0,len(messages)):
			if messages[beforeIter] == messages[Iter] and Iter != beforeIter:
				result["isExist"] = True
				result["spams"].append([beforeIter, Iter])
			beforeIter = Iter
		return result

	class lockSchedule:
		def __init__(self): pass

		def setLockTime(self, h, m):
			global lockTime
			lockTime = f"{h}:{m}"
	
		def setUnlockTime(self, h, m):
			global unlockTime
			unlockTime = f"{h}:{m}"

		def getLockTime(self):
			global lockTime
			return lockTime

		def getUnlockTime(self):
			global unlockTime
			return unlockTime
	
		def checkLockTime(self, guid, accesses=[]):
			global lockTime
			if datetime.now().strftime("%H:%M") == lockTime:
				bot.setMembersAccess(guid, accesses)
	
		def checkUnlockTime(self, guid, accesses=["ViewMembers","ViewAdmins","SendMessages","AddMember"]):
			global lockTime
			if datetime.now().strftime("%H:%M") == lockTime:
				bot.setMembersAccess(guid, accesses)

	def commandHandler(self, message, command, method:callable):
		if message == command: method()

	def isJoin(self, user_guid:str, channel_guid:str) -> bool:
		userInfo       = bot.getInfo(user_guid)
		haveUserName   = 'username' in userInfo.keys()
		channelMembers = lambda keyword : bot.getMembers(channel_guid, search_text=keyword) if keyword != None else []

		if haveUserName: result = [True for i in channelMembers(userInfo.get('username'))     if i['member_guid'] == user_guid][0]
		else:
			result = [True for i in channelMembers(userInfo.get('first_name'))   if i['member_guid'] == user_guid][0]
			result = [True for i in channelMembers(userInfo.get('last_name'))    if i['member_guid'] == user_guid][0]

		return result == True

	def faster(self, controller):
		def get(): socket.handle()
		def use(): controller(socket.data)
		
		while True:
			Thread(target=get).start()
			Thread(target=use).start()

	loadTime = lambda timestamp, func=__import__("jdatetime").datetime.fromtimestamp: func(int(timestamp))

	@staticmethod
	def parse(mode, text):
		results = []
		
		realText = text.replace("**","").replace("__","").replace("`","").replace("[","").replace("]","").replace("(","").replace(")","") if mode.lower() == "markdown" else text.replace("<b>","").replace("</b>","").replace("<i>","").replace("</i>","").replace("<pre>","").replace("</pre>","").replace("<a href='","").replace("'>","").replace("</a>", "")
		bolds    = findall(r"\*\*(.*?)\*\*",text) if mode.lower() == "markdown" else findall("<b>(.*?)</b>",text)
		italics  = findall("__(.*?)__",text) if mode.lower() == "markdown" else findall("<i>(.*?)</i>",text)
		monos    = findall("`(.*?)`",text) if mode.lower() == "markdown" else findall("<pre>(.*?)</pre>",text)
		Mentions = findall("\[(.*?)\]\((.*?)\)", text) if mode.lower() == "markdown" else findall("<a href='(.*?)'>(.*?)</a>", text)

		bResult = [realText.index(i) for i in bolds]
		iResult = [realText.index(i) for i in italics]
		mResult = [realText.index(i) for i in monos]
		MResult = [(realText.index(i),realText.index(j)) for i,j in Mentions]

		for i in findall(r"\w.{31}", realText): realText = realText.replace(i, "")

		for index,word,Type in ((bResult, bolds, ["Bold"]*len(bolds)), (iResult, italics, ["Italic"]*len(italics)), (mResult, monos, ["Mono"]*len(monos)), (MResult, Mentions, ["MentionText"]*len(Mentions))):
			if index != [] and word != []:
				result = {"from_index": index[0] if Type[0] != "MentionText" else index[0][0],"length": len(word[0] if Type[0] != "MentionText" else word[0][1]),"type": Type[0]}
				if Type[0] == "MentionText": result["mention_text_object_guid"], result["mention_text_object_type"] = word[0][0], "User"
				results.append(result)
		
		return dict(metadata=results, realText=realText)

	@staticmethod
	def getThumbInline(image_bytes:bytes) -> str:
		import io, base64, PIL.Image
		im, output = PIL.Image.open(io.BytesIO(image_bytes)), io.BytesIO()
		width, height = im.size
		im.save(output, format='PNG')
		im_data = output.getvalue()
		image_data = base64.b64encode(im_data)
		if not isinstance(image_data, str): image_data = image_data.decode()
		return image_data

	@staticmethod
	def getImageSize(image_bytes:bytes) -> list:
		import io, PIL.Image
		im = PIL.Image.open(io.BytesIO(image_bytes))
		width, height = im.size
		return [width , height]


import base64,urllib3
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

class encryption:
	def __init__(self, auth): self.key, self.iv = bytearray(self.secret(auth), "UTF-8"), bytearray.fromhex('00000000000000000000000000000000')

	def secret(self, e):
		t, n, s = e[0:8], e[16:24]+e[0:8]+e[24:32]+e[8:16], 0
		while s < len(n):
			t = chr((ord(n[s][0]) - ord('0') + 5) % 10 + ord('0')) if n[s] >= '0' and n[s] <= '9' else chr((ord(n[s][0]) - ord('a') + 9) % 26 + ord('a'))
			n, s = self.replaceCharAt(n, s, t), s+1
		return n

	replaceCharAt = lambda self, e, t, i: e[0:t] + i + e[t + len(i):]
	encrypt       = lambda self, text: base64.b64encode(AES.new(self.key, AES.MODE_CBC, self.iv).encrypt(pad(text.encode('UTF-8'), AES.block_size))).decode('UTF-8')
	decrypt       = lambda self, text: unpad(AES.new(self.key, AES.MODE_CBC, self.iv).decrypt(base64.urlsafe_b64decode(text.encode('UTF-8'))), AES.block_size).decode('UTF-8')


import re
from sys import argv

from setuptools import setup, find_packages

with open("requirements.txt", encoding="utf-8") as r:
    requires = [i.strip() for i in r]

with open("rubika/client.py", encoding="utf-8") as f:
    version = re.findall(r"__version__ = \"(.+)\"", f.read())[0]

with open("README.md", encoding="utf-8") as f:
    readme = f.read()

setup(
    name="rubika",
    version=version,
    description="this is an unofficial library for making bots in rubika. using this library you can make your own rubika bot and control that",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/bahman-ahmadi/rubika",
    download_url="https://github.com/bahman-ahmadi/rubika/releases/latest",
    author="Bahman Ahmadi",
    author_email="bahmanahmadi.mail@gmail.com",
    license="LGPLv3",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: Implementation",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Internet",
        "Topic :: Communications",
        "Topic :: Communications :: Chat",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries :: Application Frameworks"
    ],
    keywords="rubika self bot api library python",
    project_urls={
        "Tracker": "https://github.com/bahman-ahmadi/rubika/issues",
        "Community": "https://t.me/rubikalib",
        "Source": "https://github.com/bahman-ahmadi/rubika",
        "Documentation": "https://rubikalib.github.io",
    },
    python_requires="~=3.5",
    packages=find_packages(),
    zip_safe=False,
    install_requires=requires
)


from asyncio import run
from random  import sample, choice
from json    import loads, dumps, decoder
from aiohttp import ClientSession, client_exceptions
from string  import ascii_lowercase, ascii_uppercase, digits

from rubika.encryption import encryption
from rubika.exceptions import *

class accesses:
	class admin:
		pin               = "PinMessages"
		newAdmin          = "SetAdmin"
		editInfo          = "ChangeInfo"
		banMember         = "BanMember"
		changeLink        = "SetJoinLink"
		editMembersAccess = "SetMemberAccess"
		deleteMessages    = "DeleteGlobalAllMessages"

	class user:
		viewMembers = "ViewMembers"
		viewAdmins  = "ViewAdmins"
		sendMessage = "SendMessages"
		addMember   = "AddMember"

class clients:
	web = {
		"app_name"    : "Main",
		"app_version" : "4.0.7",
		"platform"    : "Web",
		"package"     : "web.rubika.ir",
		"lang_code"   : "fa"
	}
	android = {
		"app_name"    : "Main",
		"app_version" : "2.9.8",
		"platform"    : "Android",
		"package"     : "app.rbmain.a",
		"lang_code"   : "fa"
	}

class reports:
	OTHER       = 100
	VIOLENCE    = 101
	SPAM        = 102
	PORNOGRAPHY = 103
	CHILD_ABUSE = 104
	COPYRIGHT   = 105
	FISHING     = 106

class visibility:
	EVERYBODY  = "Everybody"
	MYCONTACTS = "MyContacts"
	NOBODY     = "Nobody"

randStr = lambda length, choices=[*ascii_lowercase, *ascii_uppercase, *digits, *"-_"]: "".join(sample(choices, length))
randStr = lambda length, choices=[*ascii_lowercase, *ascii_uppercase, *digits, *"-_"]: "".join([choice(choices) for i in range(length)])

defaultDevice = {
	"app_version"      : "MA_2.9.8",
	"device_hash"      : randStr(32, [*ascii_uppercase, *digits]),
	"device_model"     : "rubikalib",
	"is_multi_account" : False,
	"lang_code"        : "fa",
	"system_version"   : "SDK 22",
	"token"            : f"{randStr(22, [*ascii_lowercase, *ascii_uppercase, *digits])}:{randStr(140)}",
	"token_type"       : "Firebase"
}

def post(json:dict, method, url:str=None, platform="rubika", enc:encryption=None, isEncrypted:bool=True) -> dict:
	async def POST(url, data):
		while 1:
			try:
				async with ClientSession() as session:
					async with session.post(url, json=data) as response:
						response = await response.text()
						response = loads(str(enc.decrypt(loads(response).get("data_enc")))) if "data_enc" in loads(response).keys() and isEncrypted else loads(response)
						if type(response) == dict:
							if "status" in response.keys() and response.get("status") != "OK":
								if response.get("status_det") == "NOT_REGISTERED":
									print( data, url, response )
									raise NotRegistered("the auth is incorrect. please sure about your account's health then login again.")
								elif response.get("status_det") == "INVALID_INPUT" :
									print( data, url, response )
									raise InvalidInput(f"the inserted argument(s) is invaild in the {platform}/{method}. if you're sure about your argument(s), please report this message.")
								elif response.get("status_det") == "TOO_REQUESTS" :
									print( data, url, response )
									raise TooRequests(f"the {platform}/{method} method has been limited. please try again later.")
								elif response.get("status_det") == 'INVALID_AUTH':
									print( data, url, response )
									raise InvaildAuth(f"the inserted argument(s) in {platform}/{method} is vaild but is not related to other argument(s) or maybe for other reasons, anyway now this method can't run on server. please don't enter fake argument(s) and fix anything can return this exception")
							else:
								return response
						elif type(response) == str and response.startswith("<!DOCTYPE"):
							post(json, method, url=url, platform=platform, enc=enc, isEncrypted=isEncrypted)
			except decoder.JSONDecodeError: ...
			except client_exceptions.ClientConnectorError: ...
	return run(POST(url or _getURL(dc_id=64), json))

def _getURL(DCsURL:str="https://messengerg2cX.iranlms.ir/", getDCsURL:str="https://getdcmess.iranlms.ir", dc_id:int=None):
	from requests import get as GET, exceptions
	while 1:
		try:
			dc_id = dc_id or choice(loads(GET(getDCsURL).text).get("data").get("default_apis"))
			return DCsURL.replace("X", str(dc_id))
		except exceptions.ConnectionError: ...

def makeData(api_version:int, auth:str, method:str, data:dict, client:dict, url:str=_getURL()) -> dict :
	enc = encryption(auth)
	outerJson = {
		"api_version" : str(api_version),
		"auth"        : auth,
		"data_enc"    : enc.encrypt(dumps({
			"method"  : method,
			"input"   : data,
			"client"  : client
		}))
	}

	if int(api_version) == 4:
		outerJson["data_enc"] = enc.encrypt(dumps(data))
		outerJson["client"]   = client
		outerJson["method"]   = method

	return post(outerJson, url=url, platform=client.get('package'), method=method, enc=enc)

def makeTmpData(method:str, data:dict, url:str=_getURL()) -> dict:
	tmp = tmpGeneration()
	enc = encryption(tmp)
	outerJson = {
		"api_version": "5",
		"tmp_session": tmp,
		"data_enc"   : enc.encrypt(dumps({
			"method" : method,
			"input"  : data,
			"client" : clients.web
		}))
	}
	
	return post(outerJson, method, url=url, platform=clients.web.get("package"), enc=enc)

def makeRubinoData(auth:str, method:str, data:dict) -> dict:
	outerJson = {
		"api_version": "0",
		"auth"   : auth,
		"client" : clients.android,
		"data"   : data,
		"method" : method
	}
	return post(outerJson, method, url="https://rubino12.iranlms.ir/", platform="rubino", isEncrypted=False)

def makeWSData(ws, auth:str, method:str, data:dict) :
	enc = encryption(auth)
	return ws.send(dumps({"api_version": "4","auth": auth,"data_enc": enc.encrypt(dumps(data)),"method": method}))

tmpGeneration = lambda: randStr(32, [*ascii_lowercase, *digits])

def welcome(text, time:float=0.035):
	if __file__.split("/")[-2] != "rubika":
		from time import sleep
		try:
			from rich import print
			print(text)
		except ModuleNotFoundError:
			for char in text:
				print(char, end='', flush=True)
				sleep(float(time))
		print()